package com.example.demo.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.example.demo.constant.EleConstant;
import com.example.demo.constant.StaConstant;
import com.example.demo.mapper.ElecBillInfoMapper;
import com.example.demo.mapper.ElecPrivceHistoryMapper;
import com.example.demo.mapper.ElectricityBillMapper;
import com.example.demo.model.*;
import com.example.demo.service.BasicService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Slf4j
@Service
public class BasicServiceImpl implements BasicService {

    @Autowired
    ElectricityBillMapper electricityBillMapper;

    @Autowired
    ElecBillInfoMapper elecBillInfoMapper;

    @Autowired
    ElecPrivceHistoryMapper elecPrivceHistoryMapper;

    @Override
    public Response getElectricityInfo(Map<String, Object> param) {
        String keyWord = (String) param.get("kw");
        Response response = new Response();
        List<ElectricityBill> bills = new ArrayList<>();
        TypeAndQuery typeAndQuery = new TypeAndQuery();
        analysisKeyWord(keyWord,typeAndQuery);
        if (0 == typeAndQuery.getQuerySymbol()){
            log.info("不支持的查询类型");
            response.setSta(StaConstant.UNSUPPORTED_QUERY_TYPE);
            response.setMessage("failed");
            response.setData("暂时还不支持查询此类问题，您可以说查询电费或查询电量");
            return response;
        }
        DataAndView dataAndView =new DataAndView();
        dataAndView.setType(typeAndQuery.getType());
        if (1 == typeAndQuery.getQuerySymbol()){
            log.info("数据库查询电量！");
            response.setSta(StaConstant.UNSUPPORTED_QUERY_TYPE);
            response.setMessage("failed");
            response.setData("电量查询暂未开放！");
            return response;
        }else if (2 == typeAndQuery.getQuerySymbol()){
            log.info("数据库查询电费！");
            bills = electricityBillMapper.queryElectricityByDate();
            dataAndView.setData(bills);
            response.setSta(StaConstant.SUCCESS);
            response.setMessage("success");
            response.setData(dataAndView);
        }else {
            log.info("暂时还没有此类查询！");
            response.setSta(StaConstant.UNSUPPORTED_QUERY_TYPE);
            response.setMessage("failed");
            response.setData("暂时还不支持查询此类问题，您可以说查询电费或查询电量");
            return response;
        }
        return response;
    }

    @Override
    public Object getElectricityInfo1(Map<String, Object> param) {
        String keyWord = (String) param.get("kw");
        DataAndView dataAndView =new DataAndView();
        List<ElectricityBill> bills = new ArrayList<>();
        TypeAndQuery typeAndQuery = new TypeAndQuery();
        analysisKeyWord(keyWord,typeAndQuery);
        dataAndView.setType(typeAndQuery.getType());
            log.info("数据库查询电费！");
            bills = electricityBillMapper.queryElectricityByDate();
            StringBuilder electricityBill =new StringBuilder();
            StringBuilder statementDate =new StringBuilder();
            bills.stream().forEach(bill ->{
                electricityBill.append(bill.getElectricityBill()).append(";");
                statementDate.append(bill.getStatementDate()).append(";");
            });
            Map<String,String> result = new HashMap<>();
            result.put("electricityBill",electricityBill.deleteCharAt(electricityBill.length()-1).toString());
            result.put("statementDate",statementDate.deleteCharAt(electricityBill.length()-1).toString());
            dataAndView.setData(result);
            return dataAndView;
    }

    @Override
    public Object getBindingInfo(ElecBindInfoModel model) {
        QueryWrapper<ElecBindInfo> queryWrapper =new QueryWrapper<>();
        Map<String,Object> map = new HashMap<>();
        map.put("phone",model.getPhone());
        map.put("province",model.getProvince());
        map.put("city",model.getCity());
        map.put("elec_number",model.getElecNumber());
        map.put("city_code",model.getCityCode());
        map.put("user_number",model.getUserNumber());
        queryWrapper.allEq(map,false);
        return elecBillInfoMapper.selectList(queryWrapper);
    }

    @Override
    public Object getElePriceInfo(ElePriceInfoModel model) {
        QueryWrapper<ElecPriceHistory> queryWrapper =new QueryWrapper<>();
        Map<String,Object> map = new HashMap<>();
        List<Map<String,String>> result = new ArrayList<>();
        map.put("is_summer",model.getIsSummer());
        map.put("elec_gradient",model.getElecGradient());
        map.put("elec_type",model.getElecType());
        queryWrapper.allEq(map,false)
                .between("effective_time",model.getStartTime(),model.getEndTime())
                .like("province",model.getProvince())
                .like("city",model.getCity());
        List<ElecPriceHistory> elecPriceHistories = elecPrivceHistoryMapper.selectList(queryWrapper);
        elecPriceHistories.stream().forEach(e ->{
            Map<String,String> priceMap = new HashMap<>();
            priceMap.put("price",e.getElecPrice());
            priceMap.put("date",e.getEffectiveTime());
            result.add(priceMap);
        });
        return result;
    }

    @Override
    public Object getElePriceFromBill(ElePriceInfoModel param) {
        // 处理时间
        String startTime = param.getStartTime().replace("-","");
        String endTime = param.getEndTime().replace("-","");
        List<ElectricityBill> electricityBills = electricityBillMapper.selectList(
                new LambdaQueryWrapper<ElectricityBill>()
                .eq(ElectricityBill::getElectricityUserNumber, param.getElecNumber())
                .between(ElectricityBill::getStatementDate, startTime, endTime));
        List<Map<String, Object>> list = electricityBills.stream().map(e -> {
            Map<String, Object> priceMap = new HashMap<>();
            priceMap.put("price", e.getImplementElectricityPrices());
            String date = e.getStatementDate().replaceAll("(\\d{4})(\\d{2})", "$1-$2");
            priceMap.put("date", date);
            return priceMap;
        }).collect(Collectors.toList());
        return list;
    }

    private void analysisKeyWord(String keyWord,TypeAndQuery typeAndQuery){
        int querySymbol = 0;
        String type = EleConstant.LINE_TYPE_BG;
        typeAndQuery.setType(type);
        typeAndQuery.setQuerySymbol(querySymbol);
        if (keyWord.contains(EleConstant.ELECTRICITY)){
            querySymbol = EleConstant.ELECTRICITY_SYMBOL;
            typeAndQuery.setQuerySymbol(querySymbol);
        }else if (keyWord.contains(EleConstant.QUANTITY_OF_ELECTRICITY)){
            querySymbol = EleConstant.QUANTITY_OF_ELECTRICITY_SYMBOL;
            typeAndQuery.setQuerySymbol(querySymbol);
        }
        for (String val:EleConstant.barList){
            if (keyWord.contains(val)){
                type = EleConstant.LINE_TYPE_ZZ;
                typeAndQuery.setType(type);
                return;
            }
        }
        for (String val:EleConstant.lineList){
            if (keyWord.contains(val)){
                type = EleConstant.LINE_TYPE_ZX;
                typeAndQuery.setType(type);
                return;
            }
        }
        for (String val:EleConstant.pieList){
            if (keyWord.contains(val)){
                type = EleConstant.LINE_TYPE_SX;
                typeAndQuery.setType(type);
                return;
            }
        }
    }
}
